mod expr;
mod statement;

use expr::{
    Expr,
    Literal,
    Variable,
    BinaryOperator,
};

use statement::Stat;
use statement::Block;
use statement::GlobalBlock;

pub trait GazType {
    fn get_base(&self) -> BaseType;
}

#[derive(Copy, Clone)]
pub enum Quantifier {
    Const,
    Var,
    Unset,
}

impl Default for Quantifier {
    fn default() -> Self {
        Quantifier::Unset
    }
}

impl ToString for Quantifier {
    fn to_string(&self) -> String {
        match *self {
            Quantifier::Const => "const",
            Quantifier::Var => "var",
            Quantifier::Unset => panic!("Found unset quantifier"),
        }.to_string()
    }
}

#[derive(Copy, Clone)]
pub enum BaseType {
    Int,
    Real,
    Never,
    Unset,
}

impl Default for BaseType {
    fn default() -> Self {
        BaseType::Unset
    }
}

impl ToString for BaseType {
    fn to_string(&self) -> String {
        // TODO: Use typedefs when possible. Might fit better on a high level node
        match *self {
            BaseType::Int => "integer",
            BaseType::Real => "real",
            BaseType::Never => panic!("Attempting to get string of never type"),
            BaseType::Unset => panic!("Found unset basetype"),
        }.to_string()
    }
}
